""" A transformation that replaces expression with others from a given dictionary """

from dune.perftool.ufl.transformations import ufl_transformation
from ufl.algorithms import MultiFunction


class ReplaceExpression(MultiFunction):
    def __init__(self, replacemap={}, otherwise=None):
        MultiFunction.__init__(self)
        self.replacemap = replacemap
        self.otherwise = otherwise

    def expr(self, o):
        if o in self.replacemap:
            return self.replacemap[o]
        else:
            if self.otherwise:
                return self.otherwise(o)
            else:
                return self.reuse_if_untouched(o, *tuple(self(op) for op in o.ufl_operands))


@ufl_transformation(name="replace")
def replace_expression(expr, **kwargs):
    return ReplaceExpression(**kwargs)(expr)
